#ifndef LOADER
//#include <stdio.h>
#endif
#include "f020.h"
#include "br_setup.h"
#include "print.h"

//#pragma codeseg  BR_SETUP_CODE
//#pragma constseg BR_SETUP_CONST

//__bit b_menu, b_exit, b_up, b_down;

#define menu P3_5
#define exit P3_4
#define up   P3_3
#define down P3_2

/* 76543210
   00MEUD00
*/
#define m_menu 0x20
#define m_ok   m_menu
#define m_exit 0x10
#define m_esc  m_exit
#define m_up   0x08
#define m_down 0x04

#define led7 P0_7
#define led6 P0_6
#define led5 P0_5
#define led4 P0_4

#define LED0 P0_7
#define LED1 P0_6
#define LED2 P0_5
#define LED3 P0_4

#define LED70 0x80
#define LED71 0x40
#define LED72 0x20
#define LED73 0x10

static void
debounc()
{
	volatile unsigned int i;
	for (i= get_Fosc()/350; i; i--) ;
}


static unsigned char key_state;

static void
setup_keys()
{
	key_state= P3;
}

static unsigned char
pressed(unsigned char mask)
{
	if ((key_state & mask) != (P3 & mask))
	{
		key_state= (key_state & ~mask) | (P3 & mask);
		debounc();
		return (P3&mask)==0;
	}
	return 0;
}

static void
display3(unsigned char x)
{
	LED0= x&1;
	LED1= x&2;
	LED2= x&4;
	P7= (P7 & ~(LED70|LED71|LED72)) | ((x&1)?LED70:0) | ((x&2)?LED71:0) | ((x&4)?LED72:0);
}

static void
display4(unsigned char x)
{
	display3(x);
	LED3= x&8;
	P7= (P7 & ~LED3) | ((x&8)?LED3:0);
}


static unsigned char selected_menu;
//static long fosc;

#ifndef LOADER
//extern pbi(struct t_board_info *);
//extern dump(unsigned char *p, int nr, int line);
#endif

void
save_board_info(long freq, unsigned char clear)
{
	CODE_PTR(struct t_board_info) bi;
	XRAM_PTR(struct t_board_info) save;
	XRAM_PTR(char) flash;
	unsigned int i;

	display4(1);
	/* Save last non-reserved page to XRAM */
	bi  = (CODE_PTR(struct t_board_info))(BOARD_INFO & ~(PAGE_SIZE-1));
	save= (XRAM_PTR(struct t_board_info))(XRAM_SIZE-PAGE_SIZE);

#ifndef LOADER
	print_fp("code %p -> xram ", bi);
	print_fp("%p\r", save);
#endif
	display4(2);
	for (i= 0; i<PAGE_SIZE-INFO_BLOCK_SIZE; i++)
		((XRAM_PTR(char))save)[i]= ((CODE_PTR(char))bi)[i];
	/* Do not copy part of INFO_BLOCK */
	for (; i<PAGE_SIZE; i++)
		((XRAM_PTR(char))save)[i]= 0xff;

	bi= (CODE_PTR(struct t_board_info))BOARD_INFO;
	if (!clear)
	{
		display4(3);
		save= (XRAM_PTR(struct t_board_info))(XRAM_SIZE - INFO_BLOCK_SIZE);

		/* Setup board_info structure in XRAM */
#ifndef LOADER
		print_fp("setup bi %p\r", save);
#endif
		display4(4);
		for (i= 0; i<NUOF_OSC; i++)
		{
			save->clk_table[i].osc= clk_table[i].osc;
			if (bi->magic_sign == MAGIC_SIGN)
				save->clk_table[i].freq=
					((i==used_clk)&&(freq!=0))?freq:bi->clk_table[i].freq;
			else
				save->clk_table[i].freq=
					((i==used_clk)&&(freq!=0))?freq:clk_table[i].freq;
		}
		save->used_br= used_br;
		save->used_clk= used_clk;
		save->magic_sign= MAGIC_SIGN;
#ifndef LOADER
//		pbi(save);
#endif
	}

	display4(5);
	/* Copy XRAM to flash */
	flash= (XRAM_PTR(char               ))(BOARD_INFO & ~(PAGE_SIZE-1));
	save = (XRAM_PTR(struct t_board_info))(XRAM_SIZE-PAGE_SIZE);

#ifndef LOADER
	print_fp("xram %p -> code \r", save);
	print_fp("%p\r", flash);
#endif
	display4(6);
	FLSCL/*|*/= 1;
	PSCTL= 3; /* Erase */
	*flash= 0;
	print_p(flash);
	print_c(' ');
	PSCTL= 1; /* Write */
	display4(7);
	
	for (i= 0; i< PAGE_SIZE; i++)
	{
		/*
		print_x(((int)flash) + i);
		serial_send(' ');
		print_cx(((__xdata char *)save)[i]);
		serial_send('\r');
		*/
		flash[i]= ((XRAM_PTR(char))save)[i];
	}

	display4(8);
	PSCTL= 0;
	FLSCL/*&= ~1;*/=0;
	display4(9);
	for (i= 0; i<sizeof(struct t_board_info); i++)
	{
		print_cx(((CODE_PTR(unsigned char))bi)[i]);
		print_c(' ');
	}
	print_c('\r');
}


static void
selected_osc_changed(unsigned char x)
{
	print_d(x);
	print(": ");
	print_d(clk_table[x].osc);
	print(" ");
	print_d(clk_table[x].freq);
	print("\r");
}

static void
do_select_osc()
{
	enum t_osc x= (enum t_osc)0;
	/*printf_small("CLK[%d]=%ld, select OSC...\r",
		used_clk, get_Fosc());*/
	print("CLK["); print_d(used_clk);print("]=");print_d(get_Fosc());print(", select OSC...\r");
	while (1)
	{
		display4(x);
		if (pressed(m_up))
		{
			if (++x>=NUOF_OSC)
				x= (enum t_osc)0;
			/*printf_small("%d: %d %ld\r", x, clk_table[x].osc, clk_table[x].freq);*/
			selected_osc_changed(x);
		}
		if (pressed(m_down))
		{
			if (x)
				x= (x-1)&15;
			else
				x= NUOF_OSC-1;
			/*printf_small("%d: %d %ld\r", x, clk_table[x].osc, clk_table[x].freq);*/
			selected_osc_changed(x);
		}
		if (pressed(m_ok))
		{
			/*printf_small("Switching to OSC=%d: %d %ld...\r",
				x, clk_table[x].osc, clk_table[x].freq);*/
			print("Switching to OSC=");
			selected_osc_changed(x);
			oscillator_init(clk_table[x].osc);
			P0^= 0xf0;
			P7^= 0xf0;
			br_init(used_br);
			/*printf_small("Done, clk[%d]=%ld\r", used_clk, get_Fosc());*/
			print("CLK["); print_d(used_clk);print("]=");print_d(get_Fosc());print("\r");
			save_board_info(0, 0);
			//return;
		}
		if (pressed(m_exit))
		{
			return;
		}
	}
}

static void
do_select_br()
{
	enum t_br b= used_br;
	while (1)
	{
		display4(b);
		if (pressed(m_up))
		{
			b= (b+1)&15;
			if (br_table[b]==0)
				b= BR_110;
		}
		if (pressed(m_down))
		{
			b= (b-1)&15;
			if (b<BR_110)
				b= BR_115K;
		}
		if (pressed(m_ok))
		{
			/*printf_small("Changing BR to=%ld (CLK=%ld)...\r",
					br_table[b], get_Fosc());*/
			print("Ch BR=");
			print_d(br_table[b]);
			print(" (CLK=");
			print_d(get_Fosc());
			print(")\r");
			br_init(b);
			P0^= 0xf0;
			P7^= 0xf0;
			/*printf_small("Done, BR=%ld\r", br_table[b]);*/
			print("Done\r");
			save_board_info(0, 0);
		}
		if (pressed(m_exit))
		{
			return;
		}
	}
}

static void
refine_br(long clk)
{
	unsigned int r;
	TR2= 0;
	r= 65536-(clk/(32*br_table[used_br]));
	RCAP2L= r&0xff;
	RCAP2H= r/256;
	TL2= RCAP2L;
	TH2= RCAP2H;
	TR2= 1;
}

static void
do_refine_br()
{
	long clk;
	unsigned char i= 0;

	clk= get_Fosc();
	while (1)
	{
		if (pressed(m_ok))
		{
			save_board_info(clk, 0);
			return;
		}
		if (pressed(m_exit))
		{
			refine_br(get_Fosc());
			return;
		}
		if (pressed(m_up))
		{
			clk+= 100000;
			refine_br(clk);
			i++;
		}
		if (pressed(m_down))
		{
			clk-= 100000;
			refine_br(clk);
			i--;
		}
		display4(i);
		{
			char c;
			/*printf_small("CLK=%ld RCAP2=%x ", clk, RCAP2H*256+RCAP2L);*/
			print("CLK=");
			print_d(clk);
			print(" RCAP2=");
			print_x(RCAP2H*256+RCAP2L);
			print_c(' ');
			for (c='A'; c<'Z'; c++)
				print_c(c);
			print_c('\r');
		}
	}
}

static void
do_sendU()
{
	unsigned char i= 0;
	while (1)
	{
		if (pressed(m_exit) || pressed(m_ok))
			return;
		print_c('U');
		i++;
		display4(i);
	}
}

/*
void
use_board_info()
{
	const struct t_board_info *bi= get_board_info();
	if (!bi)
		print("\rNo BI\r");
	else
	{
		printf_small("Use br=%d osc=%d clk=%ld\r",
			   bi->used_br, bi->used_clk, bi->clk_table[bi->used_clk].freq);
		oscillator_init(bi->clk_table[bi->used_clk].osc);
		br_init(bi->used_br);
	}
}
*/

static const char * const menu_table[]= {
	"Send \"U\"",
	"Set BR",
	"Refine BR",
	"Set OSC",
	"Clear BI",
	"Save BI",
	"",
	"Reset"
};

static void
do_menu()
{
	switch (selected_menu)
	{
		case 0: do_sendU(); break;
		case 1: do_select_br(); break;
		case 2: do_refine_br(); break;
		case 3: do_select_osc(); break;
		case 4: save_board_info(0, 1); break;
		case 5: save_board_info(0, 0); break;
		case 6: break;
		case 7: RSTSRC= 0x10;      // reset the device
				break;
	}
}

static void
print_selected_menu()
{
	print_c(selected_menu+'0');
	print(": ");
	print(menu_table[selected_menu]);
	print_c('\r');
}

void
br_setup()
{
	unsigned int i= 0;
	selected_menu= 0;
	print("\rMenu\r");
	/*printf_small("%c: %s\r", selected_menu+'0', menu_table[selected_menu]);*/
	print_selected_menu();
	while ((P3 & 0xfc) != 0xfc) ;		// Wait to release all buttons
	debounc();
	key_state= P3;
	while (1)
	{
		display3(selected_menu);
		if (pressed(m_menu))
		{
		  print("Starting ");
		  print(menu_table[selected_menu]);
		  print_c('\r');
			do_menu();
			selected_menu= 0;
			P0= P7= 0x0f;
			while ((P3 & 0xfc) != 0xfc) ; // Wait to release all buttons
			return;
		}
		if (pressed(m_exit))
		{
			selected_menu= 0;
			P0= P7= 0x0f;
			print("Exit\r");
			while ((P3 & 0xfc) != 0xfc) ; // Wait to release all buttons
			return;
		}
		if (pressed(m_up))
		{
			selected_menu= (selected_menu+1) & 7;
			/*printf_small("%c: %s\r", selected_menu+'0',
					menu_table[selected_menu]);*/
			print_selected_menu();
		}
		if (pressed(m_down))
		{
			selected_menu= (selected_menu-1) & 7;
			/*printf_small("%c: %s\r", selected_menu+'0',
					menu_table[selected_menu]);*/
			print_selected_menu();
		}
		if (i++ > 0x4000)
		{
			i= 0;
			LED3= !LED3;
			P7^= LED73;;
		}
	}
}


/* End of br_setup.c */
