#include <stdio.h>

#include "f020.h"
#include "bootld.h"
#include "serial.h"
#include "silser.h"
#include "analog.h"
#include "timer.h"
#include "t0.h"

#include "card2013.h"


/* Debounce buttons */
volatile int cnt_debounce= 0;
volatile int time_debounce= 500;
volatile bit btn_menu, btn_exit, btn_up, btn_down;


/* Handle LEDs */
volatile signed char led4_power= 100;
volatile signed char led5_power= 100;
volatile signed char led6_power= 100;
volatile signed char led7_power= 100;

volatile signed char cnt_led= 0;


/* Timer interrupt */
volatile unsigned char t0l, t0h;
volatile int sw_counter= 0;

#if defined __SDCC || defined SDCC
void t0_isr(void) __interrupt (1)
#else
interrupt void T0_int(void)
#endif
{
  TH0= t0h;
  TL0= t0l;
  /* Debounce buttons */
  if (++cnt_debounce >= time_debounce)
    {
      cnt_debounce= 0;
      btn_menu= BTN5;
      btn_exit= BTN4;
      btn_up= BTN3;
      btn_down= BTN2;
    }
  /* handle LEDs */
  if (++cnt_led > 99)
    cnt_led= 0;
  LED= (LED & ~LED_MASK) |
    ((cnt_led < led4_power)?LED4:0) |
    ((cnt_led < led5_power)?LED5:0) |
    ((cnt_led < led6_power)?LED6:0) |
    ((cnt_led < led7_power)?LED7:0);
  /* General counter */
  if (++sw_counter >= 10000)
    sw_counter= 0;
}


/* Handle button presses */
bit prev_menu, prev_exit, prev_up, prev_down;
bit press_menu= 0, press_exit= 0, press_up= 0, press_down= 0;

void
check_buttons()
{
  if (btn_menu != prev_menu)
    {
      prev_menu= btn_menu;
      if (!btn_menu)
	press_menu= 1;
    }
  if (btn_exit != prev_exit)
    {
      prev_exit= btn_exit;
      if (!btn_exit)
	press_exit= 1;
    }
  if (btn_up != prev_up)
    {
      prev_up= btn_up;
      if (!btn_up)
	press_up= 1;
    }
  if (btn_down != prev_down)
    {
      prev_down= btn_down;
      if (!btn_down)
	press_down= 1;
    }
}

unsigned char
pressed_menu()
{
  if (!press_menu)
    return 0;
  press_menu= 0;
  return 1;
}

unsigned char
pressed_exit()
{
  if (!press_exit)
    return 0;
  press_exit= 0;
  return 1;
}

unsigned char
pressed_up()
{
  if (!press_up)
    return 0;
  press_up= 0;
  return 1;
}

unsigned char
pressed_down()
{
  if (!press_down)
    return 0;
  press_down= 0;
  return 1;
}


/* Demonstrations */

void
demo0(void)
{
  if (pressed_up())
    {
      if ((led4_power+= 10) > 100)
	led4_power= -1;
      printf("LED power increased to %d\n", led4_power);
    }
  if (pressed_down())
    {
      if ((led4_power-= 10) < 0)
	led4_power= 100;
      printf("LED power decreased to %d\n", led4_power);
    }  
  led5_power= led4_power;
  led6_power= led4_power;
  led7_power= led4_power;
}

void
demo1(void)
{
  led4_power= analog_p();
  led5_power= led4_power;
  led6_power= led4_power;
  led7_power= led4_power;
}

int nuof_demo= 2;

/* Main program */
void
main(void)
{
  unsigned int t0reload;
  int demo= 0;

  silabs_init(CLK_22M);
  serial_init(9600);
  br_init(BR_19K);
  analog_init(2);

  prev_menu= btn_menu= BTN5;
  prev_exit= btn_exit= BTN4;
  prev_up= btn_up= BTN3;
  prev_down= btn_down= BTN2;

  Fosc= (float)get_Fosc()/1000000.0;
  t0reload= T_kezdoertek(0.1);
  T0idozito1(0);
  T0beallit(t0reload);
  t0l= t0reload&0xff;
  t0h= t0reload/256;
  EA= 1;
  ET0= 1;
  T0start();
  
  for (;;)
    {
      reload_code();
      check_buttons();
      switch (demo)
	{
	case 0: demo0(); break;
	case 1: demo1(); break;
	}
      if (pressed_menu())
	{
	  demo++;
	  if (demo >= nuof_demo)
	    demo= 0;
	  printf("Starting demo %d\n", demo);
	}
    }
}
