#include "f020.h"
#include "fx.h"

//#pragma codeseg  SILABS_CODE
//#pragma constseg SILABS_CONST


void
Reset_Sources_Init()
{
  WDTCN= 0xDE;
  WDTCN= 0xAD;
}


enum t_br used_br;
unsigned char used_clk;

const long
#if defined __C51__
code
#endif
br_table[]= {
  0, // 0
  110, // 1
  150, // 2
  300, // 3
  600, // 4
  1200, // 5
  2400, // 6
  4800, // 7
  9600, // 8
  19200, // 9
  38400, // 10
  57600, // 11
  115200, // 12
  0
};

const struct t_clk_table
#if defined __C51__
code
#endif
clk_table[NUOF_OSC]= {
  /*  osc  ,   freq  */
  { CLK_2M ,  2000000 },
  { CLK_4M ,  4000000 }, //4500000,
  { CLK_8M ,  8000000 }, //9000000,
  { CLK_16M, 16000000 }, //18000000,
  { CLK_11M, 11059200 },
  { CLK_22M, 22118400 }
};


#define STBI struct t_board_info
const struct t_board_info *
get_board_info()
{
  /*__code struct t_board_info * */ CODE_PTR(STBI)bi=
    (/*__code struct t_board_info * */CODE_PTR(STBI))BOARD_INFO;

  if (bi->magic_sign == MAGIC_SIGN)
    return bi;
  
  for (bi= (/*__code struct t_board_info * */CODE_PTR(STBI))(0x1000 - RESERVED_SPACE);
       bi > (/*__code struct t_board_info * */CODE_PTR(STBI))(0x10000 - RESERVED_SPACE - (2*PAGE_SIZE));
       bi= (/*__code struct t_board_info * */CODE_PTR(STBI))(((/*__code char * */CODE_PTR(char))bi) - INFO_BLOCK_SIZE)
       )
    if (bi->magic_sign == MAGIC_SIGN)
      return bi;
  
  return 0;
}

long
get_Fosc()
{
  const struct t_clk_table *ctable;
  const struct t_board_info *board_info;

  board_info= get_board_info();
  ctable= (board_info)?(board_info->clk_table):clk_table;
  
  if ((OSCICN & 0x08)!=0)
    {
      /* external */
      if ((OSCXCN & 0x10)!=0)
	return ctable[4].freq;
      else
	return ctable[5].freq;
    }
  else
    {
      /* internal */
      switch (OSCICN & 7)
	{
	case CLK_2M:  return ctable[0].freq; break;
	case CLK_4M:  return ctable[1].freq; break;
	case CLK_8M:  return ctable[2].freq; break;
	case CLK_16M: return ctable[3].freq; break;
	}
    }
  return 11059200;
}


void
br_init(enum t_br br)
{
  unsigned int r;

  TR2= 0;
  if (br!=BR_0)
    {
      r= 65536-(get_Fosc()/(32*br_table[br]));
      RCAP2L= r&0xff;
      RCAP2H= r/256;
      TL2= RCAP2L;
      TH2= RCAP2H;
      TR2= 1;
    }
  used_br= br;
}

void
Port_IO_Init()
{
  P0MDOUT   = 0xF0;
  /*P1MDOUT   = 0xFF;*/
  /*P2MDOUT   = 0xFF;*/
  /*P3MDOUT   = 0xFF;*/
  P74OUT    = 0x80; /* Card v2013, LEDs on P7.4-P7.7 */
  XBR0      = 0x04;
  XBR2      = 0x40;
  P0= P1= P2= P3= 0xff;
  P4= P5= P6= P7= 0xff;
}

void
oscillator_init(enum t_osc osc)
{
  __bit EAorg;
  int i;
	
  EAorg= EA;
  EA= 0;
  i = 0;
  if ((P3 & 0x30) == 0)
    {
      if ((osc == CLK_22M) || (osc == CLK_11M))
	osc= CLK_16M;
    }
  if ((osc==CLK_11M) || (osc==CLK_22M))
    {
      /* external */
      OSCXCN= 0x67;
      OSCXCN|= (osc==CLK_11M)?0x10:00;
      OSCICN= (OSCICN&~0x07)|4;
      for (i= 0; i < 3000; i++) ;  /* Wait 1ms for initialization */
      for (i= 0; (i<3000) && ((OSCXCN&0x80)!=0x80); i++) ;
      OSCICN= 8|4;
    }
  else
    {
      /* internal */
      OSCICN= (OSCICN&~0x07)|osc;
      for (i= 0; i < 3000; i++);  /* Wait 1ms for initialization */
      for (i= 0; (i<3000) && ((OSCICN&0x10)!=0x10); i++) ;
      OSCICN&= ~0x08;
      OSCXCN= 7;
    }
  EA= EAorg;
  for (i= 0; i<NUOF_OSC; i++)
    if (clk_table[i].osc == osc)
      used_clk= i;
}


#if defined __SDCC || defined SDCC
static __sfr __at(0x97) s97;
#elif defined __C51__
static sfr s97= 0x97;
#endif

unsigned char board= 0;

void
silabs_init(enum t_osc osc)
{
  /*__code unsigned char * */CODE_PTR(unsigned char)boot_loader= (/*__code unsigned char * */CODE_PTR(unsigned char))LOADER_START;
  void (*reprogram)();
  unsigned char p, i;
  bit p0mdout, p97;

  p= P0MDOUT;
  p0mdout= 1;
  for (i= 0; i<16; i++)
    {
      P0MDOUT= 0xf0+i;
      if (P0MDOUT!=0xf0+i)
	{
	  p0mdout= 0;
	  break;
	}
    }
  if (p0mdout)
    P0MDOUT= p;

  p= s97;
  p97= 1;
  for (i= 0; i<16; i++)
    {
      s97= 0xf0+i;
      if (s97!=0xf0+i)
	{
	  p97= 0;
	  break;
	}
    }
  if (p97)
    s97= p;

  if (p97)
    {
      /* simulator */
      board= 1;
      return;
    }
  if (!p0mdout)
    {
      /* Atmel card */
      board= 2;
      return;
    }
  
  /* Silabs card */
  board= 3;

  reprogram= (void(*)())LOADER_START;
  if ((P3 & 0x1c) == 0)
    reprogram();
  
  Port_IO_Init();
  Reset_Sources_Init();
  oscillator_init(osc);
}


/* End of sialbs.c */
