/*
 * Support of student board 2012/2013/1
 * CPU: Silabs C8051F020
 */

#ifndef F020_HEADER
#define F020_HEADER

#include "hw.h"
#include "fx.h"


#define NUOF_OSC 6

/* Clock frequencies */
enum t_osc {
  CLK_2M= 4,
  CLK_4M= 5,
  CLK_8M= 6,
  CLK_16M= 7,
  CLK_11M= 0x18,
  CLK_22M= 8
};

/* Baud-rates */
enum t_br {
  BR_0= 0,
  BR_110= 1,
  BR_150= 2,
  BR_300= 3,
  BR_600= 4,
  BR_1K= 5,
  BR_2K= 6,
  BR_4K= 7,
  BR_9K= 8,
  BR_19K= 9,
  BR_38K= 10,
  BR_56K= 11,
  BR_115K= 12
};

struct t_clk_table {
  enum t_osc osc;
  long freq;
};

#define MAGIC_SIGN 0xb0ad1df0

struct t_board_info {
  unsigned long int magic_sign;
  struct t_clk_table clk_table[6];
  enum t_br used_br;
  unsigned char used_clk;
};


extern enum t_br used_br;
extern unsigned char used_clk; // index to clk_table
extern const long
#if defined __C51__
code
#endif
br_table[];
extern const struct t_clk_table
#if defined __C51__
code
#endif
clk_table[NUOF_OSC];

extern const struct t_board_info *get_board_info();
extern void silabs_init(enum t_osc osc);
extern void br_init(enum t_br br);
extern void oscillator_init(enum t_osc osc);


#if defined __SDCC || defined SDCC
#include <mcs51/C8051F020.h>
#elif defined __C51__

/*  BYTE Registers  */
sfr P4       =  0x84; 	/* PORT 4						   */
sfr P5       =  0x85; 	/* PORT 5                                                  */
sfr P6       =  0x86; 	/* PORT 6						   */
sfr CKCON    =  0x8E;	/* CLOCK CONTROL                                           */
sfr PSCTL    =  0x8F;	/* PROGRAM STORE R/W CONTROL                               */
sfr TMR3CN   =  0x91;	/* TIMER 3 CONTROL                                         */
sfr TMR3RLL  =  0x92;	/* TIMER 3 RELOAD REGISTER - LOW BYTE                      */
sfr TMR3RLH  =  0x93;	/* TIMER 3 RELOAD REGISTER - HIGH BYTE                     */
sfr TMR3L    =  0x94;	/* TIMER 3 - LOW BYTE                                      */
sfr TMR3H    =  0x95;	/* TIMER 3 - HIGH BYTE                                     */
sfr P7	     =  0x96;   /* PORT 7                                                  */
sfr SCON0    =  0x98;	/* SERIAL PORT 0 CONTROL                                   */
sfr SBUF0    =  0x99;	/* SERIAL PORT 0 BUFFER                                    */
sfr SPI0CFG  =  0x9A;	/* SERIAL PERIPHERAL INTERFACE 0 CONFIGURATION             */
sfr SPI0DAT  =  0x9B;	/* SERIAL PERIPHERAL INTERFACE 0 DATA                      */
sfr ADC1     =  0x9C;   /* ADC 1 DATA                                              */
sfr SPI0CKR  =  0x9D;	/* SERIAL PERIPHERAL INTERFACE 0 CLOCK RATE CONTROL        */
sfr CPT0CN   =  0x9E;	/* COMPARATOR 0 CONTROL                                    */
sfr CPT1CN   =  0x9F;	/* COMPARATOR 1 CONTROL                                    */
sfr EMI0TC   =  0xA1;   /* EMIF TIMING CONTROL                                     */
sfr EMI0CF   =  0xA3;   /* EXTERNAL MEMORY INTERFACE (EMIF) CONFIGURATION          */
sfr P0MDOUT  =  0xA4;	/* PORT 0 OUTPUT MODE CONFIGURATION                        */
sfr P1MDOUT  =  0xA5;	/* PORT 1 OUTPUT MODE CONFIGURATION                        */
sfr P2MDOUT  =  0xA6;	/* PORT 2 OUTPUT MODE CONFIGURATION                        */
sfr P3MDOUT  =  0xA7;	/* PORT 3 OUTPUT MODE CONFIGURATION                        */
sfr SADDR0   =  0xA9;   /* SERIAL PORT 0 SLAVE ADDRESS                             */
sfr ADC1CN   =  0xAA;   /* ADC 1 CONTROL                                           */
sfr ADC1CF   =  0xAB;   /* ADC 1 ANALOG MUX CONFIGURATION                          */
sfr AMX1SL   =  0xAC;   /* ADC 1 ANALOG MUX CHANNEL SELECT                         */
sfr P3IF     =  0xAD;   /* PORT 3 EXTERNAL INTERRUPT FLAGS                         */
sfr SADEN1   =  0xAE;   /* SERIAL PORT 1 SLAVE ADDRESS MASK                        */
sfr EMI0CN   =  0xAF;   /* EXTERNAL MEMORY INTERFACE CONTROL                       */
sfr OSCXCN   =  0xB1;	/* EXTERNAL OSCILLATOR CONTROL                             */
sfr OSCICN   =  0xB2;	/* INTERNAL OSCILLATOR CONTROL                             */
sfr P74OUT   =  0xB5;   /* PORTS 4 - 7 OUTPUT MODE                                 */
sfr FLSCL    =  0xB6;	/* FLASH MEMORY TIMING PRESCALER                           */
sfr FLACL    =  0xB7;	/* FLASH ACESS LIMIT                                       */
sfr SADEN0   =  0xB9;   /* SERIAL PORT 0 SLAVE ADDRESS MASK                        */
sfr AMX0CF   =  0xBA;	/* ADC 0 MUX CONFIGURATION                                 */
sfr AMX0SL   =  0xBB;	/* ADC 0 MUX CHANNEL SELECTION                             */
sfr ADC0CF   =  0xBC;	/* ADC 0 CONFIGURATION                                     */
sfr P1MDIN   =  0xBD;   /* PORT 1 INPUT MODE                                       */
sfr ADC0L    =  0xBE;	/* ADC 0 DATA - LOW BYTE                                   */
sfr ADC0H    =  0xBF;	/* ADC 0 DATA - HIGH BYTE                                  */
sfr SMB0CN   =  0xC0;	/* SMBUS 0 CONTROL                                         */
sfr SMB0STA  =  0xC1;	/* SMBUS 0 STATUS                                          */
sfr SMB0DAT  =  0xC2;	/* SMBUS 0 DATA                                            */
sfr SMB0ADR  =  0xC3;	/* SMBUS 0 SLAVE ADDRESS                                   */
sfr ADC0GTL  =  0xC4;	/* ADC 0 GREATER-THAN REGISTER - LOW BYTE                  */
sfr ADC0GTH  =  0xC5;	/* ADC 0 GREATER-THAN REGISTER - HIGH BYTE                 */
sfr ADC0LTL  =  0xC6;	/* ADC 0 LESS-THAN REGISTER - LOW BYTE                     */
sfr ADC0LTH  =  0xC7;	/* ADC 0 LESS-THAN REGISTER - HIGH BYTE                    */
sfr T4CON    =  0xC9;   /* TIMER 4 CONTROL                                         */
sfr SMB0CR   =  0xCF;	/* SMBUS 0 CLOCK RATE                                      */
sfr REF0CN   =  0xD1;	/* VOLTAGE REFERENCE 0 CONTROL                             */
sfr DAC0L    =  0xD2;	/* DAC 0 REGISTER - LOW BYTE                               */
sfr DAC0H    =  0xD3;	/* DAC 0 REGISTER - HIGH BYTE                              */
sfr DAC0CN   =  0xD4;	/* DAC 0 CONTROL                                           */
sfr DAC1L    =  0xD5;	/* DAC 1 REGISTER - LOW BYTE                               */
sfr DAC1H    =  0xD6;	/* DAC 1 REGISTER - HIGH BYTE                              */
sfr DAC1CN   =  0xD7;	/* DAC 1 CONTROL                                           */
sfr PCA0CN   =  0xD8;	/* PCA 0 COUNTER CONTROL                                   */
sfr PCA0MD   =  0xD9;	/* PCA 0 COUNTER MODE                                      */
sfr PCA0CPM0 =  0xDA;	/* CONTROL REGISTER FOR PCA 0 MODULE 0                     */
sfr PCA0CPM1 =  0xDB;	/* CONTROL REGISTER FOR PCA 0 MODULE 1                     */
sfr PCA0CPM2 =  0xDC;	/* CONTROL REGISTER FOR PCA 0 MODULE 2                     */
sfr PCA0CPM3 =  0xDD;	/* CONTROL REGISTER FOR PCA 0 MODULE 3                     */
sfr PCA0CPM4 =  0xDE;	/* CONTROL REGISTER FOR PCA 0 MODULE 4                     */
sfr XBR0     =  0xE1;	/* DIGITAL CROSSBAR CONFIGURATION REGISTER 0               */
sfr XBR1     =  0xE2;	/* DIGITAL CROSSBAR CONFIGURATION REGISTER 1               */
sfr XBR2     =  0xE3;	/* DIGITAL CROSSBAR CONFIGURATION REGISTER 2               */
sfr RCAP4L   =  0xE4;   /* TIMER 4 CAPTURE REGISTER - LOW BYTE                     */
sfr RCAP4H   =  0xE5;   /* TIMER 4 CAPTURE REGISTER - HIGH BYTE                    */
sfr EIE1     =  0xE6;	/* EXTERNAL INTERRUPT ENABLE 1                             */
sfr EIE2     =  0xE7;	/* EXTERNAL INTERRUPT ENABLE 2                             */
sfr ADC0CN   =  0xE8;	/* ADC 0 CONTROL                                           */	
sfr PCA0L    =  0xE9;	/* PCA 0 TIMER - LOW BYTE                                  */	
sfr PCA0CPL0 =	0xEA;   /* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 0 - LOW BYTE  */
sfr PCA0CPL1 =	0xEB;   /* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 1 - LOW BYTE  */
sfr PCA0CPL2 =	0xEC;   /* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 2 - LOW BYTE  */
sfr PCA0CPL3 =	0xED;   /* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 3 - LOW BYTE  */
sfr PCA0CPL4 =	0xEE;   /* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 4 - LOW BYTE  */
sfr RSTSRC   =	0xEF;   /* RESET SOURCE                                            */
sfr SCON1    =  0xF1;   /* SERIAL PORT 1 CONTROL                                   */
sfr SBUF1    =  0xF2;   /* SERAIL PORT 1 DATA                                      */
sfr SADDR1   =  0xF3;   /* SERAIL PORT 1                                           */ 
sfr TL4      =  0xF4;   /* TIMER 4 DATA - LOW BYTE                                 */
sfr TH4      =  0xF5;   /* TIMER 4 DATA - HIGH BYTE                                */
sfr EIP1     =	0xF6;   /* EXTERNAL INTERRUPT PRIORITY REGISTER 1                  */
sfr EIP2     =	0xF7;   /* EXTERNAL INTERRUPT PRIORITY REGISTER 2                  */
sfr SPI0CN   =	0xF8;   /* SERIAL PERIPHERAL INTERFACE 0 CONTROL                   */	
sfr PCA0H    =	0xF9;  	/* PCA 0 TIMER - HIGH BYTE                                 */
sfr PCA0CPH0 =	0xFA;  	/* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 0 - HIGH BYTE */
sfr PCA0CPH1 =	0xFB;  	/* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 1 - HIGH BYTE */
sfr PCA0CPH2 =	0xFC;  	/* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 2 - HIGH BYTE */
sfr PCA0CPH3 =	0xFD;  	/* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 3 - HIGH BYTE */
sfr PCA0CPH4 =	0xFE;  	/* CAPTURE/COMPARE REGISTER FOR PCA 0 MODULE 4 - HIGH BYTE */	
sfr WDTCN    =	0xFF;  	/* WATCHDOG TIMER CONTROL                                  */
	

/*  BIT Registers  */

/*  SCON0  0x98 */
sbit SM00  = SCON0 ^ 7;             /* SERIAL MODE CONTROL BIT 0           */	
sbit SM10  = SCON0 ^ 6;             /* SERIAL MODE CONTROL BIT 1           */
sbit SM20  = SCON0 ^ 5;             /* MULTIPROCESSOR COMMUNICATION ENABLE */
sbit REN0  = SCON0 ^ 4;             /* RECEIVE ENABLE                      */
sbit TB80  = SCON0 ^ 3;             /* TRANSMIT BIT 8                      */
sbit RB80  = SCON0 ^ 2;             /* RECEIVE BIT 8                       */
sbit TI0   = SCON0 ^ 1;             /* TRANSMIT INTERRUPT FLAG             */
sbit RI0   = SCON0 ^ 0;             /* RECEIVE INTERRUPT FLAG              */

/*  IE  0xA8 */
sbit ES0   = IE ^ 4;                /* UART0 INTERRUPT ENABLE       */

/* SMB0CN 0xC0 */
sbit BUSY     =   SMB0CN ^ 7;       /* SMBUS 0 BUSY                    */
sbit ENSMB    =   SMB0CN ^ 6;       /* SMBUS 0 ENABLE                  */
sbit STA      =   SMB0CN ^ 5;       /* SMBUS 0 START FLAG              */
sbit STO      =   SMB0CN ^ 4;       /* SMBUS 0 STOP FLAG               */
sbit SI       =   SMB0CN ^ 3;       /* SMBUS 0 INTERRUPT PENDING FLAG  */
sbit AA       =   SMB0CN ^ 2;       /* SMBUS 0 ASSERT/ACKNOWLEDGE FLAG */
sbit SMBFTE   =   SMB0CN ^ 1;       /* SMBUS 0 FREE TIMER ENABLE       */
sbit SMBTOE   =   SMB0CN ^ 0;       /* SMBUS 0 TIMEOUT ENABLE          */

/*  T2CON  0xC8 */
sbit RCLK0 = T2CON ^ 5;             /* UART0 RX CLOCK SOURCE        */
sbit TCLK0 = T2CON ^ 4;             /* UART0 TX CLOCK SOURCE        */
sbit CT2   = T2CON ^ 1;             /* TIMER OR COUNTER SELECT      */
sbit CPRL2 = T2CON ^ 0;             /* CAPTURE OR RELOAD SELECT     */

/*  PSW  */
sbit F1    = PSW ^ 1;               /* USER FLAG 1             */

/* PCA0CN D8H */
sbit CF    =   PCA0CN ^ 7;          /* PCA 0 COUNTER OVERFLOW FLAG   */
sbit CR    =   PCA0CN ^ 6;          /* PCA 0 COUNTER RUN CONTROL BIT */
sbit CCF4  =   PCA0CN ^ 4;          /* PCA 0 MODULE 4 INTERRUPT FLAG */
sbit CCF3  =   PCA0CN ^ 3;          /* PCA 0 MODULE 3 INTERRUPT FLAG */
sbit CCF2  =   PCA0CN ^ 2;          /* PCA 0 MODULE 2 INTERRUPT FLAG */
sbit CCF1  =   PCA0CN ^ 1;          /* PCA 0 MODULE 1 INTERRUPT FLAG */
sbit CCF0  =   PCA0CN ^ 0;          /* PCA 0 MODULE 0 INTERRUPT FLAG */

/* ADC0CN E8H */
sbit AD0EN     =   ADC0CN ^ 7;      /* ADC 0 ENABLE                              */
sbit AD0TM     =   ADC0CN ^ 6;      /* ADC 0 TRACK MODE                          */
sbit AD0INT    =   ADC0CN ^ 5;      /* ADC 0 CONVERISION COMPLETE INTERRUPT FLAG */
sbit AD0BUSY   =   ADC0CN ^ 4;      /* ADC 0 BUSY FLAG                           */
sbit AD0CM1    =   ADC0CN ^ 3;      /* ADC 0 START OF CONVERSION MODE BIT 1      */
sbit AD0CM0    =   ADC0CN ^ 2;      /* ADC 0 START OF CONVERSION MODE BIT 0      */
sbit AD0WINT   =   ADC0CN ^ 1;      /* ADC 0 WINDOW COMPARE INTERRUPT FLAG       */
sbit AD0LJST   =   ADC0CN ^ 0;      /* ADC 0 RIGHT JUSTIFY DATA BIT              */

/* SPI0CN F8H */
sbit SPIF     =   SPI0CN ^ 7;       /* SPI 0 INTERRUPT FLAG			*/
sbit WCOL     =   SPI0CN ^ 6;       /* SPI 0 WRITE COLLISION FLAG	*/
sbit MODF     =   SPI0CN ^ 5;       /* SPI 0 MODE FAULT FLAG		*/
sbit RXOVRN   =   SPI0CN ^ 4;       /* SPI 0 RX OVERRUN FLAG		*/
sbit TXBSY    =   SPI0CN ^ 3;       /* SPI 0 TX BUSY FLAG			*/
sbit SLVSEL   =   SPI0CN ^ 2;       /* SPI 0 SLAVE SELECT			*/
sbit MSTEN    =   SPI0CN ^ 1;       /* SPI 0 MASTER ENABLE			*/
sbit SPIEN    =   SPI0CN ^ 0;       /* SPI 0 SPI ENABLE				*/			

/* Fake SDCC */
sbit P0_0= P0^0;
sbit P0_1= P0^1;
sbit P0_2= P0^2;
sbit P0_3= P0^3;
sbit P0_4= P0^4;
sbit P0_5= P0^5;
sbit P0_6= P0^6;
sbit P0_7= P0^7;

#else /* IAR4 */
#pragma language=extended

/*  BYTE Registers  */
sfr         P0 = (0x80);
sfr         SP = (0x81);
sfr        DPL = (0x82);
sfr        DPH = (0x83);
sfr         P4 = (0x84);
sfr         P5 = (0x85);
sfr         P6 = (0x86);
sfr       PCON = (0x87);
sfr       TCON = (0x88);
sfr       TMOD = (0x89);
sfr        TL0 = (0x8A);
sfr        TL1 = (0x8B);
sfr        TH0 = (0x8C);
sfr        TH1 = (0x8D);
sfr      CKCON = (0x8E);
sfr      PSCTL = (0x8F);
sfr         P1 = (0x90);
sfr     TMR3CN = (0x91);
sfr    TMR3RLL = (0x92);
sfr    TMR3RLH = (0x93);
sfr      TMR3L = (0x94);
sfr      TMR3H = (0x95);
sfr         P7 = (0x96);
sfr       SCON = (0x98);
sfr      SCON0 = (0x98);
sfr       SBUF = (0x99);
sfr      SBUF0 = (0x99);
sfr    SPI0CFG = (0x9A);
sfr    SPI0DAT = (0x9B);
sfr       ADC1 = (0x9C);
sfr    SPI0CKR = (0x9D);
sfr     CPT0CN = (0x9E);
sfr     CPT1CN = (0x9F);
sfr         P2 = (0xA0);
sfr     EMI0TC = (0xA1);
sfr     EMI0CF = (0xA3);
sfr     PRT0CF = (0xA4);
sfr    P0MDOUT = (0xA4);
sfr     PRT1CF = (0xA5);
sfr    P1MDOUT = (0xA5);
sfr     PRT2CF = (0xA6);
sfr    P2MDOUT = (0xA6);
sfr     PRT3CF = (0xA7);
sfr    P3MDOUT = (0xA7);
sfr         IE = (0xA8);
sfr     SADDR0 = (0xA9);
sfr     ADC1CN = (0xAA);
sfr     ADC1CF = (0xAB);
sfr     AMX1SL = (0xAC);
sfr       P3IF = (0xAD);
sfr     SADEN1 = (0xAE);
sfr     EMI0CN = (0xAF);
sfr     _XPAGE = (0xAF);
sfr         P3 = (0xB0);
sfr     OSCXCN = (0xB1);
sfr     OSCICN = (0xB2);
sfr     P74OUT = (0xB5);
sfr      FLSCL = (0xB6);
sfr      FLACL = (0xB7);
sfr         IP = (0xB8);
sfr     SADEN0 = (0xB9);
sfr     AMX0CF = (0xBA);
sfr     AMX0SL = (0xBB);
sfr     ADC0CF = (0xBC);
sfr     P1MDIN = (0xBD);
sfr      ADC0L = (0xBE);
sfr      ADC0H = (0xBF);
sfr     SMB0CN = (0xC0);
sfr    SMB0STA = (0xC1);
sfr    SMB0DAT = (0xC2);
sfr    SMB0ADR = (0xC3);
sfr    ADC0GTL = (0xC4);
sfr    ADC0GTH = (0xC5);
sfr    ADC0LTL = (0xC6);
sfr    ADC0LTH = (0xC7);
sfr      T2CON = (0xC8);
sfr      T4CON = (0xC9);
sfr     RCAP2L = (0xCA);
sfr     RCAP2H = (0xCB);
sfr        TL2 = (0xCC);
sfr        TH2 = (0xCD);
sfr     SMB0CR = (0xCF);
sfr        PSW = (0xD0);
sfr     REF0CN = (0xD1);
sfr      DAC0L = (0xD2);
sfr      DAC0H = (0xD3);
sfr     DAC0CN = (0xD4);
sfr      DAC1L = (0xD5);
sfr      DAC1H = (0xD6);
sfr     DAC1CN = (0xD7);
sfr     PCA0CN = (0xD8);
sfr     PCA0MD = (0xD9);
sfr   PCA0CPM0 = (0xDA);
sfr   PCA0CPM1 = (0xDB);
sfr   PCA0CPM2 = (0xDC);
sfr   PCA0CPM3 = (0xDD);
sfr   PCA0CPM4 = (0xDE);
sfr        ACC = (0xE0);
sfr       XBR0 = (0xE1);
sfr       XBR1 = (0xE2);
sfr       XBR2 = (0xE3);
sfr     RCAP4L = (0xE4);
sfr     RCAP4H = (0xE5);
sfr       EIE1 = (0xE6);
sfr       EIE2 = (0xE7);
sfr     ADC0CN = (0xE8);
sfr      PCA0L = (0xE9);
sfr   PCA0CPL0 = (0xEA);
sfr   PCA0CPL1 = (0xEB);
sfr   PCA0CPL2 = (0xEC);
sfr   PCA0CPL3 = (0xED);
sfr   PCA0CPL4 = (0xEE);
sfr     RSTSRC = (0xEF);
sfr          B = (0xF0);
sfr      SCON1 = (0xF1);
sfr      SBUF1 = (0xF2);
sfr     SADDR1 = (0xF3);
sfr        TL4 = (0xF4);
sfr        TH4 = (0xF5);
sfr       EIP1 = (0xF6);
sfr       EIP2 = (0xF7);
sfr     SPI0CN = (0xF8);
sfr      PCA0H = (0xF9);
sfr   PCA0CPH0 = (0xFA);
sfr   PCA0CPH1 = (0xFB);
sfr   PCA0CPH2 = (0xFC);
sfr   PCA0CPH3 = (0xFD);
sfr   PCA0CPH4 = (0xFE);
sfr      WDTCN = (0xFF);

/*  WORD/DWORD Registers  */
/*
sfr       TMR0 = (0x8C8A);
sfr       TMR1 = (0x8D8B);
sfr       TMR2 = (0xCDCC);
sfr      RCAP2 = (0xCBCA);
sfr       TMR3 = (0x9594);
sfr     TMR3RL = (0x9392);
sfr       TMR4 = (0xF5F4);
sfr      RCAP4 = (0xE5E4);
sfr       ADC0 = (0xBFBE);
sfr     ADC0GT = (0xC5C4);
sfr     ADC0LT = (0xC7C6);
sfr       DAC0 = (0xD3D2);
sfr       DAC1 = (0xD6D5);
sfr       PCA0 = (0xF9E9);
sfr    PCA0CP0 = (0xFAEA);
sfr    PCA0CP1 = (0xFBEB);
sfr    PCA0CP2 = (0xFCEC);
sfr    PCA0CP3 = (0xFDED);
sfr    PCA0CP4 = (0xFEEE);
*/

/*  BIT Registers  */

/*  P0  0x80 */
bit       P0_0 = (0x80);
bit       P0_1 = (0x81);
bit       P0_2 = (0x82);
bit       P0_3 = (0x83);
bit       P0_4 = (0x84);
bit       P0_5 = (0x85);
bit       P0_6 = (0x86);
bit       P0_7 = (0x87);

/*  TCON  0x88 */
bit        IT0 = (0x88);
bit        IE0 = (0x89);
bit        IT1 = (0x8A);
bit        IE1 = (0x8B);
bit        TR0 = (0x8C);
bit        TF0 = (0x8D);
bit        TR1 = (0x8E);
bit        TF1 = (0x8F);

/*  P1  0x90 */
bit       P1_0 = (0x90);
bit       P1_1 = (0x91);
bit       P1_2 = (0x92);
bit       P1_3 = (0x93);
bit       P1_4 = (0x94);
bit       P1_5 = (0x95);
bit       P1_6 = (0x96);
bit       P1_7 = (0x97);

/*  SCON  0x98 */
bit         RI = (0x98);
bit        RI0 = (0x98);
bit         TI = (0x99);
bit        TI0 = (0x99);
bit        RB8 = (0x9A);
bit       RB80 = (0x9A);
bit        TB8 = (0x9B);
bit       TB80 = (0x9B);
bit        REN = (0x9C);
bit       REN0 = (0x9C);
bit        SM2 = (0x9D);
bit       SM20 = (0x9D);
bit       MCE0 = (0x9D);
bit        SM1 = (0x9E);
bit       SM10 = (0x9E);
bit        SM0 = (0x9F);
bit       SM00 = (0x9F);
bit     S0MODE = (0x9F);

/*  P2  0xA0 */
bit       P2_0 = (0xA0);
bit       P2_1 = (0xA1);
bit       P2_2 = (0xA2);
bit       P2_3 = (0xA3);
bit       P2_4 = (0xA4);
bit       P2_5 = (0xA5);
bit       P2_6 = (0xA6);
bit       P2_7 = (0xA7);

/*  IE  0xA8 */
bit        EX0 = (0xA8);
bit        ET0 = (0xA9);
bit        EX1 = (0xAA);
bit        ET1 = (0xAB);
bit        ES0 = (0xAC);
bit         ES = (0xAC);
bit        ET2 = (0xAD);
bit         EA = (0xAF);

/*  P3  0xB0 */
bit       P3_0 = (0xB0);
bit       P3_1 = (0xB1);
bit       P3_2 = (0xB2);
bit       P3_3 = (0xB3);
bit       P3_4 = (0xB4);
bit       P3_5 = (0xB5);
bit       P3_6 = (0xB6);
bit       P3_7 = (0xB7);

/*  IP  0xB8 */
bit        PX0 = (0xB8);
bit        PT0 = (0xB9);
bit        PX1 = (0xBA);
bit        PT1 = (0xBB);
bit        PS0 = (0xBC);
bit         PS = (0xBC);
bit        PT2 = (0xBD);

/*  SMB0CN  0xC0 */
bit     SMBTOE = (0xC0);
bit     SMBFTE = (0xC1);
bit         AA = (0xC2);
bit         SI = (0xC3);
bit        STO = (0xC4);
bit        STA = (0xC5);
bit      ENSMB = (0xC6);
bit       BUSY = (0xC7);

/*  T2CON  0xC8 */
bit      CPRL2 = (0xC8);
bit        CT2 = (0xC9);
bit        TR2 = (0xCA);
bit      EXEN2 = (0xCB);
bit       TCLK = (0xCC);
bit       RCLK = (0xCD);
bit       EXF2 = (0xCE);
bit        TF2 = (0xCF);

/*  PSW  0xD0 */
bit          P = (0xD0);
bit         F1 = (0xD1);
bit         OV = (0xD2);
bit        RS0 = (0xD3);
bit        RS1 = (0xD4);
bit         F0 = (0xD5);
bit         AC = (0xD6);
bit         CY = (0xD7);

/*  PCA0CN  0xD8H */
bit       CCF0 = (0xD8);
bit       CCF1 = (0xD9);
bit       CCF2 = (0xDA);
bit       CCF3 = (0xDB);
bit       CCF4 = (0xDC);
bit         CR = (0xDE);
bit         CF = (0xDF);

/*  ADC0CN  0xE8H */
bit     ADLJST = (0xE8);
bit    AD0LJST = (0xE8);
bit     ADWINT = (0xE9);
bit    AD0WINT = (0xE9);
bit     ADSTM0 = (0xEA);
bit     AD0CM0 = (0xEA);
bit     ADSTM1 = (0xEB);
bit     AD0CM1 = (0xEB);
bit     ADBUSY = (0xEC);
bit    AD0BUSY = (0xEC);
bit     ADCINT = (0xED);
bit     AD0INT = (0xED);
bit      ADCTM = (0xEE);
bit      AD0TM = (0xEE);
bit      ADCEN = (0xEF);
bit      AD0EN = (0xEF);

/*  SPI0CN  0xF8H */
bit      SPIEN = (0xF8);
bit      MSTEN = (0xF9);
bit     SLVSEL = (0xFA);
bit      TXBSY = (0xFB);
bit     RXOVRN = (0xFC);
bit       MODF = (0xFD);
bit       WCOL = (0xFE);
bit       SPIF = (0xFF);

/* Predefined SFR Bit Masks */
#define PCON_IDLE         0x01    /* PCON                                */
#define PCON_STOP         0x02    /* PCON                                */
#define PCON_SMOD0        0x80    /* PCON                                */
#define TF3               0x80    /* TMR3CN                              */
#define CPFIF             0x10    /* CPTnCN                              */
#define CPRIF             0x20    /* CPTnCN                              */
#define CPOUT             0x40    /* CPTnCN                              */
#define TR4               0x04    /* T4CON                               */
#define TF4               0x80    /* T4CON                               */
#define ECCF              0x01    /* PCA0CPMn                            */
#define PWM               0x02    /* PCA0CPMn                            */
#define TOG               0x04    /* PCA0CPMn                            */
#define MAT               0x08    /* PCA0CPMn                            */
#define CAPN              0x10    /* PCA0CPMn                            */
#define CAPP              0x20    /* PCA0CPMn                            */
#define ECOM              0x40    /* PCA0CPMn                            */
#define PWM16             0x80    /* PCA0CPMn                            */
#define PORSF             0x02    /* RSTSRC                              */
#define SWRSF             0x10    /* RSTSRC                              */
#define RI1               0x01    /* SCON1                               */
#define TI1               0x02    /* SCON1                               */
#define RB81              0x04    /* SCON1                               */
#define TB81              0x08    /* SCON1                               */
#define REN1              0x10    /* SCON1                               */
#define SM21              0x20    /* SCON1                               */
#define SM11              0x40    /* SCON1                               */
#define SM01              0x80    /* SCON1                               */
#endif


#endif
