#include <stdio.h>
#include <ctype.h>

#include "hw.h"
#include "f020.h"
#include "bootld.h"
#include "serial.h"
#include "print.h"

#include "timer.h"
#include "t0.h"
#include "t1.h"


#if defined SDCC || defined __SDCC
#define KI P0_4
#define pf printf_fast_f
#elif defined __C51__
sbit KI= P0^4;
#define pf printf
#else
#define KI P0.4
#define pf printf
#endif


volatile unsigned int x0, x1;
volatile char xl0, xh0, xl1, xh1;

char r= 1;

/* 100Hz-es negyszogjel a P0.4-en */
#if defined SDCC || defined __SDCC
void T0_int(void) __interrupt(1)
#elif defined __C51__
void T0_int(void) interrupt 1
#else /* IAR4 */
interrupt using[1] void T0_int(void)
#endif
{
  	/*T0ujratolt(x);*/
  	KI= !KI;
	if (KI)
	{
  		TH0= xh1;
  		TL0= xl1;
	}
	else
	{
		TH0= xh0;
		TL0= xl0;
	}
}

volatile int overflows;

#if defined SDCC || defined __SDCC
void T1_int(void) __interrupt(3)
#elif defined __C51__
void T1_int(void) interrupt 3
#else /* IAR4 */
interrupt using[3] void T1_int(void)
#endif
{
	overflows++;
}

void
calc()
{
	x1= T_kezdoertek(0.5 * r);
	x0= T_kezdoertek(0.5 * (10-r));
  	xl0= x0&0xff;
  	xh0= x0/256;
  	xl1= x1&0xff;
	xh1= x1/256;
}

void
wait_event(bool rise)
{
	bit prev= KI, found= 0;
	while (!found)
	{
		if (KI!=prev)
		{
			prev= KI;
			found= rise?KI:!KI;
		}
		reload_code();
	}
}

float
measure(bool start, bool stop)
{
	float f;

	T1idozito1(0);
	T1beallit(0);
	overflows= 0;
	ET1= 1;
	wait_event(start);
	T1start();
	wait_event(stop);
	T1stop();
	printf("%d %d\n\r", overflows, TH1*256+TL1);
	f= ((12.0/Fosc)*0x10000*overflows)/1000.0 + T1eltelt();
	pf("%f msec\n\r", f);
	return f;
}

void main(void)
{
  float t1, t2, t3, t4;
  
  //beallitas();
  silabs_init(CLK_22M);
  serial_init(9600);
  br_init(BR_19K);
/*
	for (;;)
	{
		if (vetel_volt())
			kikuld(vett());
		kikuld('U');
	}
*/
  T0idozito1(FALSE);
  Fosc= get_Fosc()/1000000.0;
  r= 1;
  calc();
  KI= 0;
  T0beallit(x0);
  ET0= 1;
  EA= 1;

  t1= T0eltelt_us();
  t2= T0hatravan_us();
  t3= T0eltelt();
  t4= T0hatravan();
  pf("eltelt: %fus/%fms, hatravan: %fus/%fms\n\r", t1, t3, t2, t4);  
  T0start();

  for (;;)
    {
      reload_code();
	  if (serial_received())
	  {
	  	char c;
	  	c= serial_send(serial_receive());
		if (isdigit(c))
		{
			r= c-'0';
			T0stop();
			calc();
			T0start();
		}
		if (c=='\n' || c=='\r')
		{
			float f;
			print("\n\r");
			print("H: ");
			measure(1, 0);
			print("L: ");
			measure(0, 1);
			print("Period: ");
			f= measure(1, 1);
			print("Freq: ");
			pf("%f Hz\n\r", 1/(f/1000.0));
		}
	  }
    }
}
